;-------- DATADICT.SC-------- Full Data Dictionary ---------------------------
;
;
;  Alan Zenreich 2/87      Revision 3.0
;  Extension of groundwork laid by:
;     Steve and James Kocis, and of course Rick Alber
;
; Procedures to produce a Data dictionary menu that allows the creation of,
; viewing of, and reporting on the records listing the current fields,
; structure and Valchecks for a selected group of tables in a directory.
;
; To run this script, the DATADICT table and reports R1 thru R8 must be in the
; current directory.  The reports are explained in the menus.
;
; The code is in script form, and due to it being heavily commented, you might
; consider writing the procedures to a library, since libraries strip out the
; comments, and it might consume a little less memory.  Although it would load
; a little faster, I doubt that you would see much of a performance increase
; everything is already proceduralized.  I have kept variable assignment to a
; minimum, since Paradox does not release the variable pool memory back to the
; central pool when asked to do so.  Also note that variables are kept private
; their procedures, which avoids confusion, since the variables release
; automatically.
;
; A good data dictionary is essential to developers to help keep track of the
; attributes assigned to fields in tables for an application.  This utility
; takes the pain out of creating the data dictionary, and since you can send
; reports to screen, as well as to the printer, I think you'll use it often.
;
; By comparing fields in reports with different sort orders, inconsistancies
; in field length, type, and valchecks become obvious.
;
; This revision adds a couple of counters so the user can track the % complete
; at any point during the creation of the records for the table.  It should be
; useful in estimating the amount of time necessary to completely load the
; table.  I always try to keep the operator informed/entertained.
;
; I hope you can follow the code.  Enjoy yourself.
;
; PS. A late note: A strange phenomenon is occurring.  It seems that if you
;  run a script that include lines like
;    EMPTY "DATADICT"
;    MENU {Tools} {Info} {Inventory} {Tables} {}
;  the list table will not include DATADICT or any other tables EMPTY'd during
;  the execution of that script.  This manifests itself in this script when you
;  are asked to empty an existing DATADICT table before adding new records.
;  When the LIST of tables is presented, DATADICT will not be among them.
;  In this case it is not a problem, but it certainly is curious.  Again, this
;  problem only exists for the duration of a script.   Ansa has been advised of
;  this anomaly, and hopefully there is a fix or workaround that I haven't
;  discovered yet.
;
;==============================================================================
   ;--Display message while procs load

CLEARALL
CLEAR
??"Loading DATA DICTIONARY procedures" STYLE BLINK ??"....." STYLE

IF NOT ISTABLE("DATADICT") THEN
  RETURN "The DATADICT table is not present, correct the situation then try again."
ENDIF

;==============================================================================
   ;--Beeps until operator presses any key
PROC beeper(delay)
PRIVATE zzzz                     ;-- zzzz any characters in the buffer

 WHILE CHARWAITING()              ;-- first CLEAR buffer if necessary
   zzzz=getchar()                 ;-- grab keystroke from buffer assign to zzzz
 ENDWHILE                         ;-- loops if more characters in buffer

 IF delay>30000 THEN              ;-- check for valid sleep time in milliseconds
    delay=1000                    ;-- if more than 30000, set delay to one second
 ENDIF

 WHILE NOT CHARWAITING()          ;-- waits for keystroke from operator
   BEEP BEEP                      ;-- two quick beeps (nice touch)
   SLEEP delay                    ;-- pauses for the amount specified by delay
 ENDWHILE                         ;-- loops if there is no character in buffer

 zzzz=GETCHAR()                   ;-- retreives the operator keystroke from
                                  ;-- the buffer, so it doesn't get passed on to
                                  ;-- program.
ENDPROC                          ;-- beeper(delay)

;==============================================================================
     ;--Makes sure printer is ready, if not gives user option to quit
PROC CHECK_PRINTER()
PRIVATE choice
 STYLE REVERSE
 @24,0
 ?? FORMAT("W80,ac","Checking Printer...")
 STYLE
 WHILE NOT PRINTERSTATUS()
   STYLE REVERSE
   @24,0
   ?? FORMAT("W80,ac","Printer is not ready, press any key for options..")
   STYLE
   BEEPER(1000)
   @24,0  CLEAR EOL
   SHOWMENU
     "Continue":
         "Turn on printer then make this choice to continue printing report",
     "Quit":"Do not print report"
   TO choice
   SWITCH
     CASE choice="Quit" or choice="Esc":
       STYLE REVERSE
       @24,0 ?? FORMAT("W80,AC","Cancelling report printing... ")
       STYLE
       RETURN FALSE
     CASE choice="Continue":
       STYLE REVERSE
       @24,0
       ?? FORMAT("W80,ac","Checking printer.....")
       STYLE
   ENDSWITCH
  ENDWHILE
 @24,0  CLEAR EOL
 RETURN TRUE
ENDPROC   ; CHECK_PRINTER()

;==============================================================================
    ;--Lets user view existing records in DATADICT in either form or table view
PROC view_dd()
PRIVATE  choice
 IF ISEMPTY("DATADICT") THEN   ;-- No records present, return to menu
   @1,0 CLEAR EOL @0,0 CLEAR EOL
   ??"The DATADICT table is empty, Choose Create from the menu, press any key..."
   beeper(1000)
   RETURN
 ENDIF
 WHILE TRUE
   CLEAR
   @23,0 STYLE REVERSE
   ??FORMAT("W80,ac","View Data Dictionary Menu")
   STYLE
   @2,0  ??FILL("",80)
   @4,0
   TEXT
Each record in the DATADICT table represents a field.  Since fields often
occur in multiple tables, viewing the actual records and their attributes
can be useful in addition to the reports.

Due to limited paper width, Y or N is sufficient for REQUIRED, and some reports
only show the first 30 characters of some of the ValChecks (this is usually
more than enough).  Viewing the DATADICT shows the entire ValChecks as Paradox
sees them.  No editing is permitted.  DATADICT is in Table Name, Field#  order.

Fields:       Field Name    - Full field name
              Field Type    - Full type including * for key field
              Field#        - Equivalent to FIELDNO()
              Table Name    - Table name
              Required      - ValChecks Required Yes or No
              Tablelookup   - ValChecks TableLookup table name if any
              Default       - ValChecks Default value if any
              High          - ValChecks High value if any
              Low           - ValChecks Low value if any
              Picture       - ValChecks Picture if any
   ENDTEXT
   @0,0
   SHOWMENU     ;-- Give user a choice
     "TableView":"View Data Dictionary in table view",
     "FormView":"View Data Dictionary in form view",
     "Return":"RETURN to Data Dictionary menu"
   TO choice
   @0,0 CLEAR EOL ??"Working" STYLE BLINK ??"...." STYLE
   IF choice="Esc" or choice="Return" THEN
     RETURN
   ENDIF
   VIEW "DATADICT"       ;-- view current records
   IF choice="FormView" THEN FORMKEY ENDIF
   WAIT TABLE
   PROMPT "VIEWING DATADICT.  "+STRVAL(nrecords("DATADICT"))+" records on file.",
          "Press F2 when done viewing table."
   UNTIL "F2"
   CLEARIMAGE            ;-- remove datadict image
 ENDWHILE
ENDPROC ;VIEW_dd

;==============================================================================
    ;--Present menu of available reports, and output devices.  Output report.
PROC report_dd()
PRIVATE  choice
 @0,0 CLEAR EOL ??"Working" STYLE BLINK ??"...." STYLE
 IF ISEMPTY("DATADICT") THEN      ;-- No records to report on
   @1,0 CLEAR EOL  @0,0 CLEAR EOL
   ??"The DATADICT table is empty, Choose Create from the menu, press any key..."
   beeper(1000)
   RETURN
 ENDIF


 CLEAR
 @23,0 STYLE REVERSE
 ??FORMAT("W80,ac","Data Dictionary REPORT Menu")
 STYLE
 @2,0  ??FILL("",80)
 @4,0
 TEXT
Reports range from a simple listing of all field names, to reports with
different sort orders.

The reports that include ALL ValChecks are 2 pages wide, and marked
page Left/page Right so pages can be placed in a notebook with pairs of pages
facing each other.

The reports can be sent to the printer or to screen.
Due to paper width limitations some of the ValChecks are truncated in some
reports.  If you need to see the full values, use the View option from the main
menu, to see the records in their entirety.
   ENDTEXT
WHILE TRUE
   @0,0
   SHOWMENU
     "1" : "List field names & number of tables that field is found in.",
     "2" : "List field names & the tables that they are found in.",
     "3" : "List fields & all ValChecks. Sorted by Table, Field#. 2 pages wide.",
     "4" : "List fields & limited ValChecks. Sorted by Table, Field#. 1 page wide.",
     "5" : "List fields & all ValChecks. Sorted by Table, Field name. 2 pages wide.",
     "6" : "List fields & limited ValChecks. Sorted by Table, Field name. 1 page wide.",
     "7" : "List fields & all ValChecks. Sorted by Field name, table. 2 pages wide.",
     "8" : "List fields & Limited ValChecks. Sorted by Field name, table. 1 page wide.",
     "Return" : "Return to the Data Dictionary menu"
   TO Choice
   IF choice="Esc" or choice="Return" THEN
     RETURN
   ENDIF
   SHOWMENU    ;--Give user a choice of output device
     "Printer":"Send Report #"+choice+" to the printer",
     "Screen":"Send Report #"+choice+" to the screen"
   TO output
   @0,0 CLEAR EOL ??"Working" STYLE BLINK ??"...." STYLE
   SWITCH
     CASE output="Screen":          ;-- Output report to screen
       MENU {Report} {Output} {DATADICT} SELECT choice {Screen}
     CASE output="Printer":
       IF check_printer() THEN      ;-- Printer is online and ready
         REPORT "DATADICT" choice   ;-- Output report to printer
       ENDIF
   ENDSWITCH
 ENDWHILE
ENDPROC ;report_dd

;==============================================================================
   ;--- Allows user to pause the loading of the DATADICT table, if paused,
   ;    then a menu offers options to continue or empty the current DATADICT.
   ;    It returns a TRUE if there are no characters in the buffer.  The proc
   ;    is called at several points in the load_dd() procedure.
PROC pause_load_dd()
PRIVATE choice
 IF NOT CHARWAITING() THEN   ;-- No key has been pressed
   RETURN TRUE               ;-- OK to continue loading DATADICT
 ENDIF

 WHILE CHARWAITING()    ;-- CLEAR the buffer of all keys pressed
  choice=GETCHAR()
 ENDWHILE
 BEEP BEEP
 SHOWMENU
   "Continue":"Continue adding fields to the Data Dictionary",
   "Quit":"Cancel all entries to, and empty the Data dictionary"
 TO choice

 IF choice="Continue" or choice="Esc" THEN     ;-- Resume loading data
   RETURN TRUE
 ENDIF
   ;-- Cleanup temporary files and empty DATADICT
 @0,1 CLEAR EOL  @0,0 CLEAR EOL
 ??"Deleting temporary files and emptying the Data Dictionary......"
 IF SYSMODE()="Edit" THEN CANCELEDIT ENDIF
 DELETE "LIST"
 IF ISTABLE("DDSTRUCT") THEN  DELETE "DDSTRUCT" ENDIF
 IF ISTABLE("STRUCT")   THEN  DELETE "STRUCT"   ENDIF
 EMPTY "DATADICT"
 CLEARALL
 RETURN FALSE
ENDPROC  ;pause_load_dd

;==============================================================================
    ;-- Makes sure DATADICT is empty befor loading data
PROC setup_dd()
PRIVATE choice ,transfer
 @0,0 CLEAR EOL ??"Working" STYLE BLINK ??"..." STYLE
 IF ISEMPTY("DATADICT") THEN
   RETURN TRUE
 ENDIF
 SHOWMENU        ;-- Give user the option to save existing table
   "Empty":"Remove the "+STRVAL(nrecords("DATADICT"))+" records in the DATADICT table",
   "Transfer":"Transfer the current contents of DATADICT to another table, then EMPTY DATADICT",
   "Return":"Return to Data dictionary menu"
 TO choice
 SWITCH
  CASE choice="Esc" or choice="Return":
    RETURN FALSE
  CASE choice="Empty":
    SHOWMENU
      "Cancel":"Do not EMPTY the DATADICT table",
      "OK":"OK to EMPTY the DATADICT table"
    TO choice
    IF choice="Esc" or choice="Cancel" THEN       ;-- decided no to empty DATADICT
       RETURN FALSE
    ENDIF
    @0,0 CLEAR EOL ??"Working..."
    EMPTY "DATADICT"
    RETURN TRUE            ;-- OK to load data
  CASE choice="Transfer":
    @1,0 CLEAR EOL  @0,0 CLEAR EOL
    ?? "Name of table to transfer DATADICT records to: "
    ACCEPT "A8" PICTURE "*!" TO transfer
    IF ISTABLE(transfer) THEN    ;-- check for existing table
      SHOWMENU
        "Cancel":"Do not overwrite existing "+upper(transfer)+" table",
        "Replace":"Replace existing "+upper(transfer)+" table with records form DATADICT"
      TO choice
      IF choice="Esc" or choice="Cancel" THEN
        RETURN FALSE
      ENDIF
    ENDIF
    @0,0 CLEAR EOL ??"Working..."
    COPY "DATADICT" transfer      ;-- Overwrites table if necessary
    EMPTY "DATADICT"
    RETURN TRUE
 ENDSWITCH

ENDPROC ;setup_dd
;==============================================================================
  ;-- Presents user with a list of available tables.  User then deletes
  ;   unwanted tables.
  ;   The LIST table is then scanned for each record (table name).
  ;   For each table :
  ;    A STRUCT table is produced and renamed DDSTRUCT
  ;    DDSTRUCT is restructured to the same structure as DATADICT
  ;    The DDSTRUCT table is scanned for each fieldname
  ;    For each record in DDSTRUCT ( field names ):
  ;      All of the ValChecks  are gotten from the table and placed in DDSTRUCT
  ;      The table name and Field# are placed into each DDSTRUCT record
  ;   If there was no operator intervention, then DDSTRUCT is added to DATADICT
  ;   There are messages displayed showing the status of the work in progress

PROC load_dd()
PRIVATE tablename,ntables,numfields,thisfield,fieldsdone

 IF NOT setup_dd() THEN  ;-- User decided to quit, or table is missing
   RETURN
 ENDIF
 IF ISTABLE("DDSTRUCT") THEN   ;-- Clean up any leftover DDSTRUCT
   DELETE "DDSTRUCT"
 ENDIF
 IF ISTABLE("STRUCT") THEN
   DELETE "STRUCT"             ;-- Clean up any leftover STRUCT
 ENDIF
 CLEARALL
 MENU {Tools} {Info} {Inventory} {Tables} {} ;-- Obtain LIST of tables
 MENU {image} {columnsize}   ;-- shrink table name column to more manageable size
 Right Enter Home Right Right Right Right Right Right Right Right Right Enter

     ;--- Allow removal of tables not wanted
 BEEP                  ;-- alert user for input
 WHILE TRUE            ;-- select tables loop
   IF NOT SYSMODE()="Edit" THEN EDITKEY ENDIF
   WAIT TABLE
   PROMPT "Use <Del> to remove any tables not wanted for the Data Dictionary Table",
          "When finished, press F2, or Esc to cancel selections and return to menu"
   UNTIL "F2","Esc"
   IF retval="Esc" THEN          ;-- clean up
     CANCELEDIT
     DELETE "LIST"
     RETURN
   ENDIF
   Home     ;-- check first record
   IF NOT ISBLANK([name]) THEN QUITLOOP ENDIF    ;-- List table is OK
   @1,0 CLEAR EOL @0,0 CLEAR EOL
   ??"OOPS. You removed all of the tables. Press any key to try again."
   beeper(1000)
   CANCELEDIT   ;--Restore full list table
 ENDWHILE             ;-- select tables loop
 DO_IT!       ;-- End edit of LIST

   ;-- display several messages to user, so he/she can track the action
   ;-- get total number of fields for data dictionary
 CLEAR
 @0,0 ??"Working" STYLE BLINK ??"...." STYLE
 @1,0 ??"To pause, press any key and wait.  You will have the options to continue or quit"
 @2,0 ??FILL("\196",80)       ;-- Draw line
 numfields=0
 thisfield=0
 fieldsdone=0
 SCAN   ;-- thru the LIST table
   numfields=numfields+NFIELDS([Name])  ;-- add # of fields for current table
 ENDSCAN
 ntables=STRVAL(nrecords("list"))    ;-- number of tables selecte
 @4,0 ?? "You selected "+ntables+" tables with a total of "+
          STRVAL(numfields)+ " fields for the Data Dictionary"

    ;-- Load the DATADICT table
 SCAN     ;-- thru LIST table , each record is a table name
   IF NOT pause_load_dd() THEN   ;-- allow operator intervention
     RETURN
   ENDIF

   fieldsdone=(NRECORDS("DATADICT"))
   tablename=[name]

   @6,0 CLEAR EOL      ;-- Display % complete
   ??STRVAL(INT((fieldsdone/numfields)*100))+"% Complete.  "
   @7,0 CLEAR EOL      ;-- Display # fields done so far
   ?? STRVAL(RECNO()-1)+ " tables with "+STRVAL(fieldsdone)+" fields done so far."
   @9,0 ??"Working on field:"
   @10,0 CLEAR EOS     ;-- Display table number
   ??"Table "+STRVAL(RECNO())+" of "+ntables   ;-- calculate
   @12,0 CLEAR EOL ??"Getting structure of "+upper(tablename)
   MENU {Tools} {Info} {Structure} SELECT [name]  ;-- Get structure for table
   IF NOT pause_load_dd() THEN   ;-- allow operator intervention
    RETURN
   ENDIF
   Rename "STRUCT" "DDSTRUCT"  ;--Can't modify a table called STRUCT
   MENU {Modify} {Restructure} {DDSTRUCT}
   End Down     ;-- prepare to add new fields
   "FIELD#" Enter  "s"  Enter
   "TABLE NAME" Enter "a8"  Enter
   "LOWVALUE" Enter  "a52" Enter
   "HIGHVALUE" Enter "a52" Enter
   "DEFAULT" Enter  "a52" Enter
   "TABLELOOKUP" Enter  "a8" Enter
   "PICTURE" Enter  "a52" Enter
   "REQUIRED" Enter  "a3" Enter
   Do_It!   ;-- Complete the restructure of DDSTRUCT
     ;-- DDSTRUCT for the table tablename is on the workspace
   IF NOT pause_load_dd() THEN   ;-- allow operator intervention
     RETURN
   ENDIF
   EDIT tablename     ;-- Prepare to get valchecks
   MOVETO "DDSTRUCT"   ;-- Next SCAN opreates from within DDSTRUCT
   @13,0 CLEAR EOL ??"Getting ValChecks for "
   SCAN      ;-- thru the DDSTRUCT table
     @13,22 CLEAR EOL ??[field name]       ;-- Display current field name
     @9,18 ??STRVAL(fieldsdone+RECNO())    ;-- Display current field so far
     IF NOT pause_load_dd() THEN   ;-- allow operator intervention
       RETURN
     ENDIF
     [table name]=tablename    ;-- Assign DDSTRUCT record the correct tablename
     [Field#]=RECNO()          ;-- RECNO corresponds to Field number
      ;-- move to the proper field in the displayed table
     EXECUTE "MOVETO ["+tablename+" -> "+[field name]+"]"
      ;-- Get all valchecks for field, assign them to DDSTRUCT record
     MENU {valcheck} {define} Enter    ;-- still in edit mode
     {lowvalue} [DDSTRUCT -> lowvalue]=MENUCHOICE() Esc  ;-back to Valcheck menu
     {highvalue} [DDSTRUCT -> highvalue]=MENUCHOICE() Esc
     {default} [DDSTRUCT -> default]=MENUCHOICE() Esc
     {Tablelookup} [DDSTRUCT -> tablelookup]=MENUCHOICE() Esc
     {Picture} [DDSTRUCT -> picture]=MENUCHOICE() Esc
     {Required} [DDSTRUCT -> required]=MENUCHOICE() Esc
     CtrlBreak   ;-- get out of valchecks definition
     MOVETO "DDSTRUCT"  ;continue the SCAN
   ENDSCAN      ;-- SCANs thru the DDSTRUCT table
   Do_It!   ;--take table back to VIEW
   MOVETO tablename
   CLEARIMAGE      ;-- Remove table that we just worked with
   @14,0 CLEAR EOL ??"Adding fields to Data Dictionary"
   IF NOT pause_load_dd() THEN   ;-- allow operator intervention
     RETURN
   ENDIF
   ADD "DDSTRUCT" "DATADICT"     ;-- Append existing records
   CLEARIMAGE                    ;-- removes DATADICT
   DELETE "DDSTRUCT"             ;-- cleanup
 ENDSCAN        ;-- SCANs thru the list table

 DELETE "list"  ;-- all done, clean up

ENDPROC; load_dd

;==============================================================================
     ;-- Main menu, gives user options to create, view and report on DATADICT

PROC MENU_dd()
PRIVATE choice
 WHILE TRUE        ;-- menu loop
   CLEAR
   @23,0 STYLE REVERSE
   ??FORMAT("W80,ac","Data Dictionary MAIN Menu")
   STYLE
   @2,0  ??FILL("",80)
   @4,0
   TEXT
Welcome to the Data Dictionary.

The DATADICT table is restricted to the current directory.

Create: Allows you to create a new DATADICT table, selecting the tables to use
        from a list.  If there are currently records in DATADICT, you are given
        the options of emptying the table, or transfer to another table before
        emptying.
Report: You can choose from 8 reports varying in detail, and sort order.
        Reports can be sent to the printer or screen.
  View: Lets you see each record in DATADICT either in form or table view.


This script and reports written by Alan Zenreich,  Feb/1987.

The development of this dictionary was inspired by the work done independantly
by Steve and James Kocis, and Rick Alber.  I hope other developers will read the
commented source code, use it, and feel free to build upon it if necessary.
   ENDTEXT
   @0,0
   SHOWMENU         ; Beginning of MENU named ddMENU
     "Create" : "Choose tables to use, and load fields into the Data Dictionary",
     "Report" : "Send any of the Data Dictionary reports to the screen of printer",
     "View" : "View the records currently in the Data Dictionary",
     "Quit" : "Return to Paradox"
   TO Choice
   SWITCH
     CASE Choice = "Esc":BEEP BEEP
     CASE Choice = "Create":
       load_dd()
     CASE Choice = "Report":
       report_dd()
     CASE Choice = "View":
       view_dd()
     CASE Choice = "Quit":
       RETURN
   ENDSWITCH
 ENDWHILE     ;-- menu loop
ENDPROC ;MENU_dd

;==============================================================================
  ;-- execute the data dictionary !!
  ;

MENU_dd()     ;-- enjoy yourself, it is incredibly useful


